//
// AHBulk.h
//
// $Revision: 1.7 $ $Date: 2007/10/19 18:12:52 $
//
// Copyright (C) 2007 Robert J. McDonald.  All rights reserved.
// http://www.dancingkayak.com/codesamples/anagrams/
//
// Loads all dictionary words in bulk into memory in one allocation and read.
// Doesn't create individual DictionaryEntry objects for each dictionary entry.
//

#ifndef AHBULK_H
#define AHBULK_H

#include "DictionaryEntry.h"
#include "AnagramHunter.h"

class AHBulk : public AnagramHunter
{
private:

	enum HuntStatus
	{
		HUNT_NOT_STARTED = 0,
		HUNT_STARTED = 1,
		HUNT_PROCEEDING = 2
	};

	struct wordInfo
	{
		char *		text;		// Ptr to null-terminated string of word
		unsigned int	length;		// Length of the string
	};

	char*			m_wordbuffer;		// Massaged contents of word list file
	unsigned int		m_wordbufferLength;	// Length of m_wordbuffer
	struct wordInfo *	m_wordlist;		// Pointers to starts of words within m_wordbuffer and lengths
	int*			m_freqTables;		// Array of m_wordcount arrays of int[LETTER_COUNT] for words pointed at in m_wordlist
	unsigned int		m_wordcount;
	unsigned int		m_entryCount;
	unsigned int		m_entriesLength;

	DictionaryEntry		m_dictEntry;
	unsigned int		m_currentIndex;
	HuntStatus		m_huntStatus;

public:

	AHBulk();

	virtual ~AHBulk();

	// Install a list of words within which to search for anagrams
	unsigned int setDict(const char * dictPath);

	// Specify the word for which anagrams will be searched
	void setCandidate(const char * candidate);

	// Find first/next anagram of the candidate
	const char * getAnagram(void);

private:

	unsigned int readDictFile(const char * dictPath);
	unsigned int countEntries(void);
	unsigned int convertDict(void);

	// Prevent compiler from auto-building unneeded copy constructor and assignment operator
	AHBulk& operator=(AHBulk& ahbulk)
	{
		return ahbulk;
	}

	AHBulk(AHBulk& ahbulk)
	{
	}
};

#endif // AHBULK_H

