//
// AHSingles.cpp
//
// $Revision: 1.9 $ $Date: 2007/10/19 18:12:52 $
//
// Copyright (C) 2007 Robert J. McDonald.  All rights reserved.
// http://www.dancingkayak.com/codesamples/anagrams/
//
// Uses individually created DictionaryEntry objects
// for each word list entry.
// This is pretty slow as a result.
//

#include <fstream>

#include "AHSingles.h"

AHSingles::AHSingles() : m_entries(NULL), m_entryCount(0), m_entriesLength(0)
{
}

AHSingles::~AHSingles()
{
	if (m_entries != NULL)
	{
		int	entryIndex;

		for (entryIndex = 0; entryIndex < m_entriesLength; entryIndex++)
		{
			if (m_entries[entryIndex] != NULL)
			{
				delete(m_entries[entryIndex]);
				m_entries[entryIndex] = NULL;
			}
		}
		::free(m_entries);
		m_entries = NULL;
	}
}

// Set the word for which anagrams will be searched
// Find first anagram
void AHSingles::setCandidate(const char * candidate)
{
	if (candidate == NULL)
	{
		m_errorMessage = "candidate is NULL";
		m_dictEntry.setEntry("");
	}
	else
	{
		m_errorMessage = "";
		m_dictEntry.setEntry(candidate);
	}
	m_currentIndex = -1;
	m_anagramCount = 0;
	return;
}

// Find the first/next anagram in the current search
const char * AHSingles::getAnagram(void)
{
	const char * nextWord = NULL;

	if (m_currentIndex < m_entryCount)
	{
		m_currentIndex++;
			
		while (m_currentIndex < m_entryCount)
		{
			DictionaryEntry*	currentEntry;

			currentEntry = m_entries[m_currentIndex];
			// Don't check the same word as the candidate
			if (0 != ::strcasecmp(currentEntry->word(), m_dictEntry.word()) &&
				0 == currentEntry->compare(m_dictEntry))
			{
				nextWord = currentEntry->word();
				m_anagramCount++;
				break;
			}
			m_currentIndex++;
		}
	}
	return nextWord;
}

unsigned int AHSingles::setDict(const char * dictPath)
{
	unsigned int	entryCount;
	std::ifstream	dictionary;
	std::string	fileLine;
	int		returnCode;

	m_anagramCount = 0;
	m_errorMessage = "";
	returnCode = 0;

	dictionary.open(dictPath, std::ios::in);

	if (dictionary.is_open())
	{
		// Read through file to count how many entries
		entryCount = 0;
		while (!dictionary.eof())
		{
			getline(dictionary, fileLine);
			// If line is not blank...
			if (fileLine.length() > 0)
			{
				entryCount++;
			}
		}
		dictionary.clear();

		// Allocate array
		m_entries = (DictionaryEntry**) calloc(entryCount, sizeof(DictionaryEntry*));
		if (m_entries == NULL)
		{
			m_errorMessage = "Can't allocate dictionary entry collection";
			returnCode = 2;
		}
		else
		{
			m_entriesLength = entryCount;

			// Rewind file to beginning
			dictionary.seekg(0, std::ios::beg);

			// Create entry for each line of file
			unsigned int entryIndex = 0;
			while (!dictionary.eof() && entryIndex < entryCount)
			{
				getline(dictionary, fileLine);
				// If line is not blank...
				if (fileLine.length() > 0)
				{
					m_entries[entryIndex] = new DictionaryEntry();
					if (m_entries[entryIndex] == NULL)
					{
						m_errorMessage = "Can't allocate DictionaryEntry. ";
						break;
					}
					m_entries[entryIndex]->setEntry(fileLine.c_str());
					entryIndex++;
				}
			}
			if (entryIndex != entryCount)
			{
				if (m_errorMessage.size() == 0)
				{
					m_errorMessage = "Dictionary entry collection length different from entry count";
				}
				returnCode = 4;
			}
			else
			{
				m_entryCount = entryCount;
			}
		}
		dictionary.close();
	}
	else
	{
		m_errorMessage = "Can't open file ";
		m_errorMessage += dictPath;
		returnCode = 1;
	}
	return returnCode;
}

