//
// DictionaryEntry.cpp
//
// $Revision: 1.6 $ $Date: 2007/10/19 18:12:52 $
//
// Copyright (C) 2007 Robert J. McDonald.  All rights reserved.
// http://www.dancingkayak.com/codesamples/anagrams/
//

#include "DictionaryEntry.h"

#include <stdlib.h>
#include <string.h>
#include <ctype.h>

DictionaryEntry::DictionaryEntry(void)
{
}

void DictionaryEntry::setEntry(const char * word)
{
	unsigned int	letterIndex;

	for (letterIndex = 0; letterIndex < LETTER_COUNT; letterIndex++)
	{
		m_freqTable[letterIndex] = 0;
	}

	// Validate and capture copy of the word
	if (word != NULL)
	{
		m_word = word;

		// Validate word
		for (letterIndex = 0; letterIndex < m_word.size(); letterIndex++)
		{
			char testLetter;

			testLetter = tolower(m_word[letterIndex]);
			if (testLetter > 'z' || testLetter < 'a')
			{
				// Words with invalid characters don't match anything
				break;
			}
		}

		if (letterIndex == m_word.size())
		{
			// Perform frequency analysis on letters in word
			for (letterIndex = 0; letterIndex < m_word.size(); letterIndex++)
			{
				char testLetter;

				testLetter = tolower(m_word[letterIndex]);
				m_freqTable[testLetter - 'a']++;
			}
		}
	}
	// else NULL resets the entry to an empty string
	else
	{
		m_word = "";
	}
};

DictionaryEntry::~DictionaryEntry()
{
};

const char * DictionaryEntry::word(void) const
{
	return m_word.c_str();
};

// Return the count of a specific letter
// in the word associated with this entry.
// Case-insensitive.
int DictionaryEntry::frequency(const char letter) const
{
	int	letterIndex;
	char	testLetter;
	int	letterCount;

	testLetter = tolower(letter);
	if (testLetter > 'z' || testLetter < 'a')
	{
		letterCount = 0;
	}
	else
	{
		letterIndex = testLetter - 'a';
		letterCount = m_freqTable[letterIndex];
	}
	return letterCount;
};

int DictionaryEntry::matchTable(const int (* freqTable)[LETTER_COUNT]) const
{
	unsigned int	letterIndex;

	for (letterIndex = 0; letterIndex < LETTER_COUNT; letterIndex++)
	{
		if (m_freqTable[letterIndex] != (*freqTable)[letterIndex])
		{
			return 1;
		}
	}
	return 0;
};

// Compare another dictionary entry with self
// Returns 0 if same, nonzero if not
int DictionaryEntry::compare(const DictionaryEntry & test) const
{
	unsigned int	letterIndex;

	for (letterIndex = 0; letterIndex < LETTER_COUNT; letterIndex++)
	{
		if (m_freqTable[letterIndex] != test.frequency('a' + letterIndex))
		{
			return 1;
		}
	}
	return 0;
};

