//
// DictionaryEntry.h
//
// $Revision: 1.4 $ $Date: 2007/10/19 18:12:52 $
//
// Copyright (C) 2007 Robert J. McDonald.  All rights reserved.
// http://www.dancingkayak.com/codesamples/anagrams/
//

#ifndef DICTIONARY_ENTRY_H
#define DICTIONARY_ENTRY_H

#include <string>

const unsigned int LETTER_COUNT = 26;

// Holds the text of a word and the frequency count of the letters in the word

class DictionaryEntry
{
private:
	std::string	m_word;				// Holds the text of the word
	int		m_freqTable[LETTER_COUNT];	// Frequency table of letters a-z

public:

	DictionaryEntry(void);
	~DictionaryEntry();

	void setEntry(const char * word);
	const char * word(void) const;

	// Return the count of a specific letter
	// in the word associated with this entry.
	// Case-insensitive.
	int frequency(const char letter) const;

	int matchTable(const int (* freqTable)[LETTER_COUNT]) const;

	// Compare another dictionary entry with self
	// Returns 0 if same, nonzero if not
	int compare(const DictionaryEntry & test) const;

private:
	// Prevent compiler from auto-building unneeded copy constructor and assignment operator
	DictionaryEntry& operator=(DictionaryEntry& dictEntry)
	{
		return dictEntry;
	}

	DictionaryEntry(DictionaryEntry& dictEntry)
	{
	}

};
#endif // DICTIONARY_ENTRY_H

