//
// C++ sample problem solution
//
// Robert McDonald
//
// $Revision: 1.12 $ $Date: 2007/10/19 18:12:52 $
//
// Copyright (C) 2007 Robert J. McDonald.  All rights reserved.
// http://www.dancingkayak.com/codesamples/anagrams/
//

// Find entries in dictionary file that are anagrams of the input word.
//
// Anagrams are words that have the same set of letters but arranged differently,
// like players and parsley.
//
// One can calculate this by using a frequency count of the input words
// and identifying all dictionary words with the same frequency count.

#include <errno.h>

#include "AHSingles.h"
#include "AHBulk.h"

// Arguments within argv[] passed to main()
static const int ARG_COUNT = 4;
static const int ARG_PROGRAM = 0;
static const int ARG_CANDIDATE = 1;
static const int ARG_DICT = 2;
static const int ARG_TYPE = 3;

// Return codes from main()
static const int EXIT_ANAGRAMS_FOUND = 0;
static const int EXIT_USAGE = 1;
static const int EXIT_DICT_ERROR = 2;
static const int EXIT_NO_ANAGRAMS = 3;
static const int EXIT_HUNTER_ERROR = 4;

// Code types to use to solve the problem
static const int CT_INDIV_DE = 0;
static const int CT_BULK_DE = 1;
static const char * CODE_TYPE_USAGE = "Code type must be one of:  0, 1";

int findAnagrams(const char * dictFile, const char * candidate, AnagramHunter & hunter)
{
	const char *	currentAnagram;

	if (hunter.setDict(dictFile))
	{
		printf("Error setting dictionary:  %s\n", hunter.errorMessage());
		return EXIT_DICT_ERROR;
	}
	hunter.setCandidate(candidate);
	while ((currentAnagram = hunter.getAnagram())!= NULL)
	{
		printf("%s\n", currentAnagram);
	}
	return (hunter.anagramCount()) ? EXIT_ANAGRAMS_FOUND : EXIT_NO_ANAGRAMS;
}

int main(int argc, char ** argv)
{
	int		retCode;
	long		codeType;
	AnagramHunter*	hunter;

	retCode = 0;
	hunter = NULL;
	if (argc != ARG_COUNT)
	{
		printf("%s\n", "Usage:  anagram word dict type\n\tword - the word for which to search for anagrams\n\tdict - path to dictionary file\n\ttype - number indicating which code set to use");
		exit(EXIT_USAGE);
	}

	codeType = strtol(argv[ARG_TYPE], NULL, 10);

	if (codeType == 0 && (errno == EINVAL || errno == ERANGE))
	{
		printf("%s\n", CODE_TYPE_USAGE);
		exit(EXIT_USAGE);
	}

	switch (codeType)
	{
	  case CT_INDIV_DE:
		hunter = new AHSingles();
		break;

	  case CT_BULK_DE:
		hunter = new AHBulk();
		break;

	  default:
		printf("%s\n", CODE_TYPE_USAGE);
		retCode = EXIT_USAGE;
		break;
	}

	if (retCode == 0)
	{
		if (hunter == NULL)
		{
			retCode = EXIT_HUNTER_ERROR;
		}
		else
		{
			retCode = findAnagrams(argv[ARG_DICT], argv[ARG_CANDIDATE], *hunter);
			delete hunter;
			hunter = NULL;
		}
	}
	return retCode;
}

